/*
 * Assembly Language Debugger
 *
 * Copyright (C) 2000 Patrick Alken
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: c_break.c,v 1.4 2001/05/26 13:58:52 cosine Exp $
 */

#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "msg.h"
#include "print.h"
#include "symbols.h"

/*
 * libDebug includes
 */
#include "break.h"

/*
c_break()
  Set a breakpoint at given address
*/

void
c_break(int ac, char **av)

{
  unsigned long address;
  int num;
  char *endptr;

  if (ac < 2)
  {
    Print(P_COMMAND, "Syntax: break <address | symbol>");
    return;
  }

  address = strtoul(av[1], &endptr, 0);
  if ((endptr == av[1]) || (*endptr != '\0'))
  {
    int err;

    /*
     * They gave an invalid number, but it may be the name
     * of a debugging symbol
     */
    err = 0;
    address = FindSymbolAddress(av[1], &err);
    if (err)
    {
      Print(P_ERROR, MSG_INVSYM, av[1]);
      return;
    }
  }

  num = SetBreakpoint(address, 0);

  if (num == (-1))
  {
    /*
     * Most likely a ptrace error
     */
    Print(P_COMMAND, "Error occurred while setting breakpoint: %s",
      strerror(errno));
  }
  else
  {
    Print(P_COMMAND, "Breakpoint %u set for 0x%08lX",
      num,
      address);
  }
} /* c_break() */
